/*   AM-CLR for ICP-PCI Kernel module free public release
 *
 *   Werner Jaeger        email: e9026603@student.tuwien.ac.at
 *   Wilfried Kubinger    email: wk@flexaut.tuwien.ac.at
 *
 *   Institute of Flexible Automation
 *   University of Technology Vienna
 *   Austria, Europe
 *
 *   amclrReg.c  v1.1  1999/03/02
 */

/* Change History:

   - 1999/05/03 12:47:07 matthias
     ajustments to version 0.5.0 (some datatypes and variables renamed)

   - Add a "break" for the case C1_CSC_BYPASS in the switch of the
     c1_get_csc function.
 
   - Add a "udelay(10)" in the two polling loops of the 
     c1_set_fpga function.

   - In General Read/Write Handler: sizeof(u_short) used to calculate offset.

   - Add function c1_dump_regs for debugging.

   - Check in c1_linearize_[olut|ilut] if vmalloc succeeded.

 */


/*
 * Defines:
 */

/*
 * Includes:
 */

#include "icpciOS.h"	/* os specific stuff */
#include "icpciExt.h"	/* offical driver interface */
#include "icpciInt.h"	/* driver internals */
#include "amclrReg.h"	/* register definitions AM-CLR */

/*
 * Globals:
 */

extern struct
icp_global_t icp;	/* main driver structure */

#if (DEBUG_ICPCI || DEBUG_ICPCI_REG)
const char *c1_reg[] = 
{ 
  "modid   " , "fpga    " , "error   " , "camcon  " , "modcon  " ,
  "error   " , "error   " , "error   " , "error   " , "error   " ,
  "error   " , "error   " , "cscadrlo" , "cscadrhi" , "cscdat  " ,
  "error   " , "adccon  " , "rnref   " , "rpref   " , "gnref   " , 
  "gpref   " , "bnref   " , "bpref   " , "error   " , "alutadr " ,
  "blutadr " , "clutadr " , "alutdat " , "blutdat " , "clutdat " ,
  "error   " , "error   "  
};
#endif /* DEBUG_ICPCI_REG */

/*
 * Prototypes:
 */

u_short c1_float2matrix(float);
float c1_matrix2float (u_short);
int c1_check_init (u_char);
 
/*
 * General Read Handler for AM-CLR Registers
 */

/* 16 bit register access with 0xFF mask */

static u_char 
c1_read_mm (board, offset)

  u_char  board;         /* number of IC-PCI board attached to the computer */
  u_char  offset;        /* offset to register base address pointer */

{
  u_char  mask = 0xFF;
  u_short tmp_val = 0;
  u_short amcc_dummy = 0;

  if (icp.cap[board]->amcc0_bug) {
    if (icp_check_amcc (board, icp.map[board]->v_cr_base + offset /
			sizeof(u_short), ICP_AMCC_READ)) {
      ICP_DELAY(icp.cap[board]->mem_wait16);
      amcc_dummy = icp_readw (icp.map[board]->v_cr_base + offset /
			      sizeof(u_short));
    }
  }
  if (icp.cap[board]->amcc1_bug) {
    ICP_DELAY(icp.cap[board]->mem_wait16);
  }

  tmp_val = icp_readw (icp.map[board]->v_mod_base + offset);

  tmp_val &= mask;

#ifdef DEBUG_ICPCI_REG
  icp_printf("icpci[%d]:  c1-readw  %s, %#06X &(%#06X)\n",
	     board, c1_reg[offset], tmp_val, mask);
#endif

  return ((u_char)tmp_val);
}

/* 16 bit register 1 (or more) bit access */

static u_char 
c1_read_ms (board, offset, mask, shift)

  u_char  board;         /* number of IC-PCI board attached to the computer */
  u_char  offset;        /* offset to register base address pointer */
  u_char  mask;          /* read mask for register */
  u_char  shift;         /* number of bits to shift right */  

{
  u_short tmp_val = 0;
  u_short amcc_dummy = 0;

  if (icp.cap[board]->amcc0_bug) {
    if (icp_check_amcc (board, icp.map[board]->v_cr_base + offset /
			sizeof(u_short), ICP_AMCC_READ)) {
      ICP_DELAY(icp.cap[board]->mem_wait16);
      amcc_dummy = icp_readw (icp.map[board]->v_cr_base + offset /
			      sizeof(u_short));
    }
  }
  if (icp.cap[board]->amcc1_bug) {
    ICP_DELAY(icp.cap[board]->mem_wait16);
  }

  tmp_val = icp_readw (icp.map[board]->v_mod_base + offset);

  tmp_val &= mask;
  tmp_val >>= shift;

#ifdef DEBUG_ICPCI_REG
  icp_printf("icpci[%d]:  c1-readw  %s, %#1X &(%#06X) >>%2d\n",
	     board, c1_reg[offset], tmp_val, mask, shift);
#endif

  return ((u_char)tmp_val);
}
  

/*
 * General Write Handler for AM-CLR Registers
 */

/* 8 bit register access with mask=0xFF */

static void 
c1_write_mm (board, offset, value)

  u_char  board;         /* number of IC-PCI board attached to the computer */
  u_char  offset;        /* offset to register base address pointer */
  u_char  value;         /* value to set to */  

{
  u_char  mask = 0xFF;
  u_short tmp_val = 0;
  u_short amcc_dummy = 0;

  tmp_val = value & mask;

#ifdef DEBUG_ICPCI_REG
  icp_printf("icpci[%d]:  c1-writew %s, %#06X &(%#06X)\n",
	     board, c1_reg[offset], tmp_val, mask);
#endif

  if (icp.cap[board]->amcc0_bug) {
    if (icp_check_amcc (board, icp.map[board]->v_cr_base + offset /
                        sizeof(u_short), ICP_AMCC_READ)) {
      ICP_DELAY(icp.cap[board]->mem_wait16);
      amcc_dummy = icp_readw (icp.map[board]->v_cr_base + offset /
			      sizeof(u_short));
    }
  }
  if (icp.cap[board]->amcc1_bug) {
    ICP_DELAY(icp.cap[board]->mem_wait16);
  }
  
  icp_writew (tmp_val, icp.map[board]->v_mod_base + offset);
}

/* 8 bit register 1 (or more) bit access */

static void 
c1_write_ms (board, offset, mask, shift, value)

  u_char  board;         /* number of IC-PCI board attached to the computer */
  u_char  offset;        /* offset to register base address pointer */
  u_char  mask;          /* read mask for register */
  u_char  shift;         /* number of bits to shift right */
  u_char  value;         /* value to set to */

{
  u_short read_val = 0;
  u_short write_val = 0;
  u_short tmp_val = 0;
  u_short amcc_dummy = 0;

  tmp_val = (u_short)value << shift;

  if (icp.cap[board]->amcc0_bug) {
    if (icp_check_amcc (board, icp.map[board]->v_cr_base + offset /
			sizeof(u_short), ICP_AMCC_READ)) {
      ICP_DELAY(icp.cap[board]->mem_wait16);
      amcc_dummy = icp_readw (icp.map[board]->v_cr_base + offset /
			      sizeof(u_short));
    }
  }
  if (icp.cap[board]->amcc1_bug) {
    ICP_DELAY(icp.cap[board]->mem_wait16);
  }

  read_val = icp_readw (icp.map[board]->v_mod_base + offset);

  read_val &= ~mask;
  write_val = read_val | tmp_val;

#ifdef DEBUG_ICPCI_REG
  icp_printf("icpci[%d]:  c1-writew %s, %#06X->%#06X %#1X &(%#06X) <<%2d\n",
	     board, c1_reg[offset], read_val, write_val, tmp_val, mask, shift);
#endif

  if (icp.cap[board]->amcc0_bug) {
    if (icp_check_amcc (board, icp.map[board]->v_cr_base + offset /
                        sizeof(u_short), ICP_AMCC_READ)) {
      ICP_DELAY(icp.cap[board]->mem_wait16);
      amcc_dummy = icp_readw (icp.map[board]->v_cr_base + offset /
			      sizeof(u_short));
    }
  }
  if (icp.cap[board]->amcc1_bug) {
    ICP_DELAY(icp.cap[board]->mem_wait16);
  }

  icp_writew (write_val, icp.map[board]->v_mod_base + offset);
}
 

/*
 * Special Read Handler for AM-CLR Registers
 */

/* --- Module-ID Register --- */

u_char 
c1_get_rev (u_char board) {

  return (c1_read_ms (board, 
                      C1_MODID_OFFSET, 
                      C1_REV_MASK,
                      C1_REV_SHIFT));
}

u_char 
c1_get_type (u_char board) {

  return (c1_read_ms (board, 
                      C1_MODID_OFFSET, 
                      C1_TYPE_MASK,
                      C1_TYPE_SHIFT));
}

/* --- Camera Control Register --- */

u_char 
c1_get_camsel (u_char board) {

  return (c1_read_ms (board, 
                      C1_CAMCON_OFFSET, 
                      C1_CAMSEL_MASK, 
                      C1_CAMSEL_SHIFT));
}

u_char 
c1_get_svhs (u_char board) {

  return (c1_read_ms (board, 
                      C1_CAMCON_OFFSET, 
                      C1_SVHS_MASK, 
                      C1_SVHS_SHIFT));
}

u_char 
c1_get_trgen0 (u_char board) {

  return (c1_read_ms (board, 
                      C1_CAMCON_OFFSET, 
                      C1_TRGEN0_MASK,
                      C1_TRGEN0_SHIFT));
}

u_char 
c1_get_trgen1 (u_char board) {

  return (c1_read_ms (board, 
                      C1_CAMCON_OFFSET, 
                      C1_TRGEN1_MASK,
                      C1_TRGEN1_SHIFT));
}

u_char 
c1_get_trgpol (u_char board) {

  return (c1_read_ms (board, 
                      C1_CAMCON_OFFSET, 
                      C1_TRGPOL_MASK,
                      C1_TRGPOL_SHIFT));
}

/* --- Module Control Register --- */

u_char 
c1_get_synctyp (u_char board) {

  return (c1_read_ms (board, 
                      C1_MODCON_OFFSET, 
                      C1_SYNCTYP_MASK, 
                      C1_SYNCTYP_SHIFT));
}

u_char 
c1_get_pclkpol (u_char board) {

  return (c1_read_ms (board, 
                      C1_MODCON_OFFSET, 
                      C1_PCLKPOL_MASK, 
                      C1_PCLKPOL_SHIFT));
}

u_char 
c1_get_lowpass (u_char board) {

  return (c1_read_ms (board, 
                      C1_MODCON_OFFSET, 
                      C1_LOWPASS_MASK, 
                      C1_LOWPASS_SHIFT));
}

/* --- ADC Control Register --- */

u_char 
c1_get_islvl (u_char board) {
 
  return (c1_read_ms (board, 
                      C1_ADCCON_OFFSET, 
                      C1_ISLVL_MASK,
                      C1_ISLVL_SHIFT));
}

u_char 
c1_get_color (u_char board) {

  return (c1_read_ms (board, 
                      C1_ADCCON_OFFSET, 
                      C1_COLOR_MASK,
                      C1_COLOR_SHIFT));
}

u_char 
c1_get_syncsel (u_char board) {

  return (c1_read_ms (board, 
                      C1_ADCCON_OFFSET, 
                      C1_SYNCSEL_MASK,
                      C1_SYNCSEL_SHIFT));
}

/* --- Output LUT B Address Register --- */

u_char
c1_get_bypass_olut (u_char board) {

  return (c1_read_ms (board,
                      C1_BLUTADR_OFFSET,
                      C1_BYPASS_OLUT_MASK,
                      C1_BYPASS_OLUT_SHIFT));
}


/* --- Color Space Converter --- */

int 
c1_get_csc (u_char board, struct c1_csc_t *matrix) {

  int i;
  u_short data_low, data_high;

  /* set start address of CSC matrix for Auto-INC */
  c1_write_mm (board, C1_CSCADRLO_OFFSET, C1_MATRIX_LO);
  c1_write_mm (board, C1_CSCADRHI_OFFSET, C1_MATRIX_HI);

  /* get value of matrix registers */     
  for (i=0; i<9; i++) {		  
    data_low = c1_read_mm (board, C1_CSCDAT_OFFSET);
    data_high = c1_read_mm (board, C1_CSCDAT_OFFSET);
    matrix->csc_matrix[i] = c1_matrix2float ((data_high << 8) | data_low);
  }

  /* read CMD0 register and decode value and flags */
  data_low = c1_read_mm (board, C1_CSCDAT_OFFSET);  
  switch (data_low & C1_CSC_IN_MASK) {
  case C1_CSC_UMAGIN:
    matrix->csc_in = C1_CSC_UMAGIN;
    break;
  case C1_CSC_SMAGIN:
    matrix->csc_in = C1_CSC_SMAGIN;
    break;
  case C1_CSC_2COMPIN:
    matrix->csc_in = C1_CSC_2COMPIN;
    break;
  case C1_CSC_PSEUDOIN:
    matrix->csc_in = C1_CSC_PSEUDOIN;
    break;
  default:
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }  
  switch (data_low & C1_CSC_OUT_MASK) {
  case C1_CSC_UMAGOUT:
    matrix->csc_out = C1_CSC_UMAGOUT;
    break;
  case C1_CSC_SMAGOUT:
    matrix->csc_out = C1_CSC_SMAGOUT;
    break;
  case C1_CSC_2COMPOUT:
    matrix->csc_out = C1_CSC_2COMPOUT;
    break;
  default:
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }
  matrix->csc_overflow = data_low & C1_CSC_OVERFLOW_MASK;
  matrix->csc_error = (data_low & C1_CSC_ERROR_MASK) >> C1_CSC_ERROR_SHIFT;

  /* read CMD1 register and decode value */
  data_low = c1_read_mm (board, C1_CSCDAT_OFFSET);
  switch (data_low & C1_CSC_SHIFT_MASK) {
  case C1_CSC_SHIFT1:
    matrix->csc_shift = C1_CSC_SHIFT1;
    break;
  case C1_CSC_SHIFT2:
    matrix->csc_shift = C1_CSC_SHIFT2;
    break;
  case C1_CSC_SHIFT4:
    matrix->csc_shift = C1_CSC_SHIFT4;
    break;
  case C1_CSC_SHIFT8:
    matrix->csc_shift = C1_CSC_SHIFT8;
    break;
  case C1_CSC_SHIFT16:
    matrix->csc_shift = C1_CSC_SHIFT16;
    break;
  case C1_CSC_BYPASS:
    matrix->csc_shift = C1_CSC_BYPASS;
    break;
  default:
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }

  return (OK);
}


/* --- Input Lookup Table --- */

int 
c1_get_ilut (u_char board, struct c1_ilut_t *ilut) {

  int i;

  /* set start address of the right Input LUT for Auto-INC */
  switch (ilut->ilut_name) {
  case 'A':
    c1_write_mm (board, C1_CSCADRLO_OFFSET, C1_ILUTA_LO);
    c1_write_mm (board, C1_CSCADRHI_OFFSET, C1_ILUTA_HI);
    break;
  case 'B':
    c1_write_mm (board, C1_CSCADRLO_OFFSET, C1_ILUTB_LO);
    c1_write_mm (board, C1_CSCADRHI_OFFSET, C1_ILUTB_HI);
    break;
  case 'C':
    c1_write_mm (board, C1_CSCADRLO_OFFSET, C1_ILUTC_LO);
    c1_write_mm (board, C1_CSCADRHI_OFFSET, C1_ILUTC_HI);
    break;
  default:
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  }

  /* get value of Input LUT registers */
  for (i=0; i<256; i++) {
    ilut->ilut_data[i] = c1_read_mm (board, C1_CSCDAT_OFFSET);
  }

  return (OK);
}


/* --- Output Lookup Table --- */

int 
c1_get_olut (u_char board, struct c1_olut_t *olut) {

  int i, j, k=0, max;

  /* for 32kByte Output LUT A and C select the right 8kByte range */
  switch (olut->olut_section) {
  case 0:   /* Output LUT Address:     0 ... 8191  */
    i = 0;   
    max = 64;
    break;
  case 1:   /* Output LUT Address:  8191 ... 16383 */
    i = 64;
    max = 128;
    break;
  case 2:   /* Output LUT Address: 16384 ... 24575 */  
    i = 128;
    max = 192;
    break;
  case 3:   /* Output LUT Address: 24576 ... 32767 */
    i = 192;
    max = 256;
    break;
  default:
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  }    

  /* get value of the right Output-LUT registers */ 
  switch (olut->olut_name) {
  case 'A':
    /*  ALUTADR and CLUTADR together form 15Bit Output LUT A Address  */
    /* 15  14  13  12  11  10   9   8   7   6   5   4   3   2   1   0 */
    /* xx  A7  A6  A5  A4  A3  A2  A1  A0  C7  C6  C5  C4  C3  C2  C1 */
    for ( ; i<max; i++) {   
      c1_write_mm (board, C1_ALUTADR_OFFSET, i); 
      for (j=0; j<256; j+=2) {   
        c1_write_mm (board, C1_CLUTADR_OFFSET, j);
        olut->olut_data[k++] = c1_read_mm (board, C1_ALUTDAT_OFFSET);
      }
    }
    break;   
  case 'B':
    /*  BLUTADR and ALUTADR together form 11Bit Output LUT B Address  */
    /* 15  14  13  12  11  10   9   8   7   6   5   4   3   2   1   0 */
    /* xx  xx  xx  xx  xx  B2  B1  B0  A7  A6  A5  A4  A3  A2  A1  A0 */
    for (i=0; i<8; i++) {   
      c1_write_ms (board, 
                   C1_BLUTADR_OFFSET, 
                   C1_BLUTADR_MASK, 
                   C1_BLUTADR_SHIFT,
                   i);
      for (j=0; j<256; j++) {   
        c1_write_mm (board, C1_ALUTADR_OFFSET, j);
        olut->olut_data[k++] = c1_read_mm (board, C1_BLUTDAT_OFFSET);
      }
    }
    /* only first 2kByte used by Output LUT B, rest is set to zero */
    for ( ; k<8192; ) olut->olut_data[k++] = 0;
    break;
  case 'C':
    /*  CLUTADR and ALUTADR together form 15Bit Output LUT C Address  */
    /* 15  14  13  12  11  10   9   8   7   6   5   4   3   2   1   0 */
    /* xx  C7  C6  C5  C4  C3  C2  C1  C0  A7  A6  A5  A4  A3  A2  A1 */
    for ( ; i<max; i++) {  
      c1_write_mm (board, C1_CLUTADR_OFFSET, i);
      for (j=0; j<256; j+=2) {   
        c1_write_mm (board, C1_ALUTADR_OFFSET, j);
        olut->olut_data[k++] = c1_read_mm (board, C1_CLUTDAT_OFFSET);
      }
    }
    break;
  default:
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  }
  return (OK);
}


/* --- Reference Voltages --- */

int 
c1_get_ref (u_char board, struct c1_ref_t *ref) {

  /* get the negative or positive reference */
  switch (ref->ref_sign) {
  case 'N':
    /* get the negative red, green or blue reference */ 
    switch (ref->ref_color) {
    case 'R':
      ref->ref_value = 
        c1_read_ms (board, C1_RNREF_OFFSET, C1_RNREF_MASK, C1_RNREF_SHIFT); 
      break;
    case 'G':
      ref->ref_value = 
        c1_read_ms (board, C1_GNREF_OFFSET, C1_GNREF_MASK, C1_GNREF_SHIFT); 
      break;
    case 'B':
      ref->ref_value = 
        c1_read_ms (board, C1_BNREF_OFFSET, C1_BNREF_MASK, C1_BNREF_SHIFT); 
      break;
    default:
      icp.errno[board] = -EICP_NOSUPPORT;
      return (ERROR);
    }
    break;
  case 'P':
    /* get the positive red, green or blue reference */
    switch (ref->ref_color) {
    case 'R':
      ref->ref_value = 
        c1_read_ms (board, C1_RPREF_OFFSET, C1_RPREF_MASK, C1_RPREF_SHIFT); 
      break;
    case 'G':
      ref->ref_value = 
        c1_read_ms (board, C1_GPREF_OFFSET, C1_GPREF_MASK, C1_GPREF_SHIFT); 
      break;
    case 'B':
      ref->ref_value = 
         c1_read_ms (board, C1_BPREF_OFFSET, C1_BPREF_MASK, C1_BPREF_SHIFT); 
      break;
    default:
      icp.errno[board] = -EICP_NOSUPPORT;
      return (ERROR);
    }
    break;
  default:
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  }

  return (OK);
}


/* --- AM-CLR Registers --- */

void 
c1_get_regs (u_char board, struct c1_regs_t *regs) {

  regs->modid_reg = c1_read_mm (board, C1_MODID_OFFSET);

  regs->camcon_reg = c1_read_mm (board, C1_CAMCON_OFFSET);
  regs->modcon_reg = c1_read_mm (board, C1_MODCON_OFFSET);
  regs->adccon_reg = c1_read_mm (board, C1_ADCCON_OFFSET);

  regs->rnref_reg = c1_read_mm (board, C1_RNREF_OFFSET); 
  regs->rpref_reg = c1_read_mm (board, C1_RPREF_OFFSET);
 
  regs->gnref_reg = c1_read_mm (board, C1_GNREF_OFFSET); 
  regs->gpref_reg = c1_read_mm (board, C1_GPREF_OFFSET);
 
  regs->bnref_reg = c1_read_mm (board, C1_BNREF_OFFSET); 
  regs->bpref_reg = c1_read_mm (board, C1_BPREF_OFFSET);

  c1_write_mm (board, C1_CSCADRLO_OFFSET, C1_CMD0_LO);
  c1_write_mm (board, C1_CSCADRHI_OFFSET, C1_CMD0_HI);
  regs->cmd0_reg = c1_read_mm (board, C1_CSCDAT_OFFSET);
  regs->cmd1_reg = c1_read_mm (board, C1_CSCDAT_OFFSET);
}


/*
 * Special Write Handler for AM-CLR Registers
 */

/* --- Camera Control Register --- */

int 
c1_set_camsel (u_char board, u_char value) {

  switch (value) {
  case C1_CAMSEL_CAM0:
  case C1_CAMSEL_CAM1:
  case C1_CAMSEL_CAM2:
  case C1_CAMSEL_CAM3:
    c1_write_ms (board, 
                 C1_CAMCON_OFFSET, 
                 C1_CAMSEL_MASK, 
                 C1_CAMSEL_SHIFT, 
                 value);
    return (OK);
  default:
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  } 
}

int 
c1_set_svhs (u_char board, u_char value) {

  switch (value) {
  case C1_SVHS_DISABLED:
  case C1_SVHS_ENABLED:
    c1_write_ms (board, 
                 C1_CAMCON_OFFSET, 
                 C1_SVHS_MASK, 
                 C1_SVHS_SHIFT, 
                 value);
    return (OK);
  default:
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }  
}

int 
c1_set_trgen0 (u_char board, u_char value) {

  switch (value) {
  case C1_TRGEN0_DISABLED:
  case C1_TRGEN0_ENABLED:
    c1_write_ms (board, 
                 C1_CAMCON_OFFSET, 
                 C1_TRGEN0_MASK, 
                 C1_TRGEN0_SHIFT, 
                 value);
    return (OK);
  default:
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }  
}

int 
c1_set_trgen1 (u_char board, u_char value) {

  switch (value) {
  case C1_TRGEN1_DISABLED:
  case C1_TRGEN1_ENABLED:
    c1_write_ms (board, 
                 C1_CAMCON_OFFSET, 
                 C1_TRGEN1_MASK, 
                 C1_TRGEN1_SHIFT,
                 value);    
    return (OK);
  default:
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }  
}

int 
c1_set_trgpol (u_char board, u_char value) {

  switch (value) {
  case C1_TRGPOL_NOINVERT:
  case C1_TRGPOL_INVERT:
    c1_write_ms (board, 
                 C1_CAMCON_OFFSET, 
                 C1_TRGPOL_MASK, 
                 C1_TRGPOL_SHIFT,
                 value);
    return (OK);
  default:
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }  
}

/* --- Module Control Register --- */

int 
c1_set_synctyp (u_char board, u_char value) {

  switch (value) {
  case C1_SYNCTYP_CV_IN:
  case C1_SYNCTYP_CS_IN:
  case C1_SYNCTYP_SS_IN:
  case C1_SYNCTYP_CS_OUT:
  case C1_SYNCTYP_SS_OUT:
  case C1_SYNCTYP_SS_OUT_PI:
  case C1_SYNCTYP_CS_OUT_PI:
  case C1_SYNCTYP_SS_OUT_PO:
  case C1_SYNCTYP_CS_OUT_PO:
  case C1_SYNCTYP_SS_IN_PI:
    c1_write_ms (board, 
                 C1_MODCON_OFFSET, 
                 C1_SYNCTYP_MASK, 
                 C1_SYNCTYP_SHIFT,
                 value);
    return (OK);
  default:
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }  
}

int 
c1_set_pclkpol (u_char board, u_char value) {

  switch (value) {
  case C1_PCLKPOL_NOINVERT:
  case C1_PCLKPOL_INVERT:
    c1_write_ms (board, 
                 C1_MODCON_OFFSET, 
                 C1_PCLKPOL_MASK, 
                 C1_PCLKPOL_SHIFT,
                 value);
    return (OK);
  default:
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }  
}

int 
c1_set_strobe1 (u_char board, u_char value) {

  switch (value) {
  case C1_STROBE1_LOW:
  case C1_STROBE1_HIGH:
    c1_write_ms (board, 
                 C1_MODCON_OFFSET, 
                 C1_STROBE1_MASK, 
                 C1_STROBE1_SHIFT,
                 value);
    return (OK);
  default:
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }  
}

int 
c1_set_strobe2 (u_char board, u_char value) {

  switch (value) {
  case C1_STROBE2_LOW:
  case C1_STROBE2_HIGH:
    c1_write_ms (board, 
                 C1_MODCON_OFFSET, 
                 C1_STROBE2_MASK, 
                 C1_STROBE2_SHIFT,
                 value);
    return (OK);
  default:
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }  
}

int 
c1_set_lowpass (u_char board, u_char value) {

  switch (value) {
  case C1_LOWPASS_DISABLED:
  case C1_LOWPASS_ENABLED:
    c1_write_ms (board, 
                 C1_MODCON_OFFSET, 
                 C1_LOWPASS_MASK, 
                 C1_LOWPASS_SHIFT,
                 value);
    return (OK);
  default:  
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }  
}

/* --- ADC Control Register --- */

int 
c1_set_islvl (u_char board, u_char value) {

  switch (value) {
  case C1_ISLVL_125MV:
  case C1_ISLVL_50MV:
    c1_write_ms (board, 
                 C1_ADCCON_OFFSET, 
                 C1_ISLVL_MASK, 
                 C1_ISLVL_SHIFT, 
                 value);
    return (OK);
  default:
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }  
}

int 
c1_set_color (u_char board, u_char value) {

  switch (value) {
  case C1_COLOR_24TRUE:  
  case C1_COLOR_15TRUE:  
  case C1_COLOR_8TRUE:   
  case C1_COLOR_8PSEUDO: 
    c1_write_ms (board, 
                 C1_ADCCON_OFFSET, 
                 C1_COLOR_MASK, 
                 C1_COLOR_SHIFT, 
                 value);
    return (OK);
  default:
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }  
}

int 
c1_set_syncsel (u_char board, u_char value) {

  switch (value) {
  case C1_SYNCSEL_RED:
  case C1_SYNCSEL_GREEN:
  case C1_SYNCSEL_BLUE:
  case C1_SYNCSEL_EXT0:
  case C1_SYNCSEL_EXT1:
    c1_write_ms (board, 
                 C1_ADCCON_OFFSET, 
                 C1_SYNCSEL_MASK, 
                 C1_SYNCSEL_SHIFT,
                 value);
    return (OK);
  default:
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }  
}

/* --- Output LUT B Address Register --- */

int
c1_set_bypass_olut (u_char board, u_char value) {

  switch (value) {
  case C1_BYPASS_OLUT_DISABLED:
  case C1_BYPASS_OLUT_ENABLED:
    c1_write_ms (board, 
                 C1_BLUTADR_OFFSET, 
                 C1_BYPASS_OLUT_MASK, 
                 C1_BYPASS_OLUT_SHIFT,
                 value);
    return (OK);
  default:  
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }  
}


/* --- write timing into FPGA-Chip --- */

int 
c1_set_fpga (u_char board, struct c1_fpga_t *fpga) {

  int i, j;
  u_short fpga_reg, camcon_reg, modcon_reg, adccon_reg, blutadr_reg,
         rnref_reg, gnref_reg, bnref_reg, rpref_reg, gpref_reg, bpref_reg;

  /* backup registers before access fpga register */ 
  camcon_reg  = c1_read_mm (board, C1_CAMCON_OFFSET);
  modcon_reg  = c1_read_mm (board, C1_MODCON_OFFSET);
  adccon_reg  = c1_read_mm (board, C1_ADCCON_OFFSET);
  blutadr_reg = c1_read_mm (board, C1_BLUTADR_OFFSET);
  rnref_reg   = c1_read_mm (board, C1_RNREF_OFFSET); 
  gnref_reg   = c1_read_mm (board, C1_GNREF_OFFSET);  
  bnref_reg   = c1_read_mm (board, C1_BNREF_OFFSET);
  rpref_reg   = c1_read_mm (board, C1_RPREF_OFFSET);
  gpref_reg   = c1_read_mm (board, C1_GPREF_OFFSET);
  bpref_reg   = c1_read_mm (board, C1_BPREF_OFFSET);

  /* turn on reset/reprogram */
  fpga_reg = 0x04;
  c1_write_mm (board, C1_FPGA_OFFSET, fpga_reg);
  
  /* turn off reset */
  fpga_reg |= 0x02;
  c1_write_mm (board, C1_FPGA_OFFSET, fpga_reg);

  /* turn off reprogram */
  fpga_reg &= ~0x04;
  c1_write_mm (board, C1_FPGA_OFFSET, fpga_reg);

  /* wait for XINIT1 and XINIT2 to go high */
  for (i=0; i<100; i++) {
    icp_udelay (10);
    fpga_reg = c1_read_mm (board, C1_FPGA_OFFSET);
    if (0x18 == (0x18 & fpga_reg)) break;
  }
  if (0x18 != (fpga_reg & 0x18)) {
    icp.errno[board] = -EICP_MASK;
    return (ERROR); 
  }
 
  /* XDONE should be low */
  fpga_reg = c1_read_mm (board, C1_FPGA_OFFSET);
  if (0x00 != (0x04 & fpga_reg)) {
    icp.errno[board] = -EICP_MASK;
    return (ERROR);
  }
  ICP_PRINTF ("icpci[%d]: fpga_reset: i=%d; fpga_reg=0x%x\n",
	      board, i, fpga_reg);

  /* write data into Xilinx */
  for (j=0; j<6626; j++) {  
    for (i=0; i<8; i++) {
      fpga_reg = (((fpga->fpga_data[j] >> i) & 0x01) | 0x02);
      c1_write_mm (board, C1_FPGA_OFFSET, fpga_reg); 
    }
  }  

  /* wait for XDONE to go high */
  for (i=0; i<100; i++) {
    icp_udelay (10);
    fpga_reg = c1_read_mm (board, C1_FPGA_OFFSET);
    if (0x04 & fpga_reg) break;
  } 
  ICP_PRINTF ("icpci[%d]: fpga_done: i=%d; fpga_reg=0x%x\n",
	      board,  i, fpga_reg);
  if (!(0x04 & fpga_reg)) {
    icp.errno[board] = -EICP_MASK;
    return (ERROR);
  }
  
  /* setup registers to there beginning value */ 
  c1_write_mm (board, C1_CAMCON_OFFSET, camcon_reg);
  c1_write_mm (board, C1_MODCON_OFFSET, modcon_reg);
  c1_write_mm (board, C1_ADCCON_OFFSET, adccon_reg);
  c1_write_mm (board, C1_BLUTADR_OFFSET, blutadr_reg);
  c1_write_mm (board, C1_RNREF_OFFSET, rnref_reg); 
  c1_write_mm (board, C1_GNREF_OFFSET, gnref_reg);  
  c1_write_mm (board, C1_BNREF_OFFSET, bnref_reg);
  c1_write_mm (board, C1_RPREF_OFFSET, rpref_reg);
  c1_write_mm (board, C1_GPREF_OFFSET, gpref_reg);
  c1_write_mm (board, C1_BPREF_OFFSET, bpref_reg);

  return (OK);       
}


/* --- Color Space Converter --- */

int 
c1_set_csc (u_char board, struct c1_csc_t *csc) {

  int i;
  u_short tmp;

  /* value range checking */
  for (i=0; i<9; i++) {
    if ((csc->csc_matrix[i]<-4.000) || (csc->csc_matrix[i]>3.996)) {
      icp.errno[board] = -EICP_RANGE; 
      return (ERROR);
    }
  }
  switch (csc->csc_in) {
  case C1_CSC_UMAGIN:
  case C1_CSC_SMAGIN:
  case C1_CSC_2COMPIN:
  case C1_CSC_PSEUDOIN:
    break;
  default:
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }
  switch (csc->csc_out) {
  case C1_CSC_UMAGOUT:
  case C1_CSC_SMAGOUT:
  case C1_CSC_2COMPOUT:
    break;
  default:
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }
  switch (csc->csc_shift) {
  case C1_CSC_SHIFT1:
  case C1_CSC_SHIFT2:
  case C1_CSC_SHIFT4:
  case C1_CSC_SHIFT8:
  case C1_CSC_SHIFT16:
  case C1_CSC_BYPASS: 
    break;
  default:
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }

  /* set start address of CSC matrix for Auto-INC */
  c1_write_mm (board, C1_CSCADRLO_OFFSET, C1_MATRIX_LO);
  c1_write_mm (board, C1_CSCADRHI_OFFSET, C1_MATRIX_HI);

  /* set value of matrix registers */
  for (i=0; i<9; i++) {		  
    tmp = c1_float2matrix (csc->csc_matrix[i]);
    c1_write_mm (board, C1_CSCDAT_OFFSET, (u_char)tmp);
    c1_write_mm (board, C1_CSCDAT_OFFSET, (u_char)((tmp >> 8) & 0x07));
  }

  /* set CMD0 register */
  c1_write_mm (board, C1_CSCDAT_OFFSET, (csc->csc_in | csc->csc_out));

  /* set CMD1 register */
  c1_write_mm (board, C1_CSCDAT_OFFSET, csc->csc_shift);

  return (OK);
}


/* --- Input Lookup Table --- */

int 
c1_set_ilut (u_char board, struct c1_ilut_t *ilut) {

  int i;

  /* set start address of the right Input LUT for Auto-INC */
  switch (ilut->ilut_name) {
  case 'A':
    c1_write_mm (board, C1_CSCADRLO_OFFSET, C1_ILUTA_LO);
    c1_write_mm (board, C1_CSCADRHI_OFFSET, C1_ILUTA_HI);
    break;
  case 'B':
    c1_write_mm (board, C1_CSCADRLO_OFFSET, C1_ILUTB_LO);
    c1_write_mm (board, C1_CSCADRHI_OFFSET, C1_ILUTB_HI);
    break;
  case 'C':
    c1_write_mm (board, C1_CSCADRLO_OFFSET, C1_ILUTC_LO);
    c1_write_mm (board, C1_CSCADRHI_OFFSET, C1_ILUTC_HI);
    break;
  default:
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  }

  /* write Input LUT value to registers */
  for (i=0; i<256; i++) {
    c1_write_mm (board, C1_CSCDAT_OFFSET, ilut->ilut_data[i]);
  }

  return (OK);
}


/* --- Output Lookup Table --- */

int 
c1_set_olut (u_char board, struct c1_olut_t *olut) {

  int i, j, k=0, max;

  /* for 32kByte Output LUT A and C select the right 8kByte range */
  switch (olut->olut_section) {
  case 0:   /* Output LUT Address:     0 ... 8191  */
    i = 0;   
    max = 64;
    break;
  case 1:   /* Output LUT Address:  8191 ... 16383 */
    i = 64;
    max = 128;
    break;
  case 2:   /* Output LUT Address: 16384 ... 24575 */  
    i = 128;
    max = 192;
    break;
  case 3:   /* Output LUT Address: 24576 ... 32767 */
    i = 192;
    max = 256;
    break;
  default:
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  }    

  /* set Output LUT value to the right registers */
  switch (olut->olut_name) {
  case 'A':
    /*  ALUTADR and CLUTADR together form 15Bit Output LUT A Address  */
    /* 15  14  13  12  11  10   9   8   7   6   5   4   3   2   1   0 */
    /* xx  A7  A6  A5  A4  A3  A2  A1  A0  C7  C6  C5  C4  C3  C2  C1 */
    for ( ; i<max; i++) {   
      c1_write_mm (board, C1_ALUTADR_OFFSET, i);
      for (j=0; j<256; j+=2) {   
        c1_write_mm (board, C1_CLUTADR_OFFSET, j);
        c1_write_mm (board, C1_ALUTDAT_OFFSET, olut->olut_data[k++]);
      }
    }
    break;
  case 'B':
    /*  BLUTADR and ALUTADR together form 11Bit Output LUT B Address  */
    /* 15  14  13  12  11  10   9   8   7   6   5   4   3   2   1   0 */
    /* xx  xx  xx  xx  xx  B2  B1  B0  A7  A6  A5  A4  A3  A2  A1  A0 */
    for (i=0; i<8; i++) {  
      c1_write_ms (board, 
                   C1_BLUTADR_OFFSET, 
                   C1_BLUTADR_MASK,
                   C1_BLUTADR_SHIFT,
                   i);
      for (j=0; j<256; j++) {  
        c1_write_mm (board, C1_ALUTADR_OFFSET, j);
        c1_write_mm (board, C1_BLUTDAT_OFFSET, olut->olut_data[k++]);
      }
    }
    break;
  case 'C':
    /*  CLUTADR and ALUTADR together form 15Bit Output LUT C Address  */
    /* 15  14  13  12  11  10   9   8   7   6   5   4   3   2   1   0 */
    /* xx  C7  C6  C5  C4  C3  C2  C1  C0  A7  A6  A5  A4  A3  A2  A1 */
    for ( ; i<max; i++) {  
      c1_write_mm (board, C1_CLUTADR_OFFSET, i);
      for (j=0; j<256; j+=2) { 
        c1_write_mm (board, C1_ALUTADR_OFFSET, j);
        c1_write_mm (board, C1_CLUTDAT_OFFSET, olut->olut_data[k++]);
      }
    }
    break;
  default:
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  }

  return (OK);
}


/* --- Reference Voltages --- */

int 
c1_set_ref (u_char board, struct c1_ref_t *ref) {

  /* value range checking */
  if (ref->ref_value >= 64) {
    icp.errno[board] = -EICP_RANGE;  
    return (ERROR);
  }

  /* set negative or positive reference */
  switch (ref->ref_sign) {
  case 'N':
    /* set negative red, green or blue reference */
    switch (ref->ref_color) {
    case 'R':
      c1_write_ms (board, C1_RNREF_OFFSET, C1_RNREF_MASK, C1_RNREF_SHIFT,
        ref->ref_value); 
      break;
    case 'G':
      c1_write_ms (board, C1_GNREF_OFFSET, C1_GNREF_MASK, C1_GNREF_SHIFT,
        ref->ref_value); 
      break;
    case 'B':
      c1_write_ms (board, C1_BNREF_OFFSET, C1_BNREF_MASK, C1_BNREF_SHIFT,
        ref->ref_value); 
      break;
    default:
      icp.errno[board] = -EICP_NOSUPPORT;
      return (ERROR);
    }
    break;
  case 'P':
    /* set positive red, green or blue reference */
    switch (ref->ref_color) {
    case 'R':
      c1_write_ms (board, C1_RPREF_OFFSET, C1_RPREF_MASK, C1_RPREF_SHIFT,
        ref->ref_value); 
      break;
    case 'G':
      c1_write_ms (board, C1_GPREF_OFFSET, C1_GPREF_MASK, C1_GPREF_SHIFT,
        ref->ref_value); 
      break;
    case 'B':
      c1_write_ms (board, C1_BPREF_OFFSET, C1_BPREF_MASK, C1_BPREF_SHIFT,
        ref->ref_value); 
      break;
    default:
      icp.errno[board] = -EICP_NOSUPPORT;
      return (ERROR);
    }
    break;
  default:
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  }

  return (OK);
}


/*
 * Special Initialization Handler for AM-CLR Registers
 */

int 
c1_init_regs (u_char board) {

  int i;  
  
  c1_write_mm (board, C1_CAMCON_OFFSET, 0x00);
  c1_write_mm (board, C1_MODCON_OFFSET, 0x00);
  c1_write_mm (board, C1_ADCCON_OFFSET, 0x00);

  c1_write_mm (board, C1_RNREF_OFFSET, 0x00);
  c1_write_mm (board, C1_RPREF_OFFSET, 0x00);

  c1_write_mm (board, C1_GNREF_OFFSET, 0x00);
  c1_write_mm (board, C1_GPREF_OFFSET, 0x00);

  c1_write_mm (board, C1_BNREF_OFFSET, 0x00);
  c1_write_mm (board, C1_BPREF_OFFSET, 0x00);

  c1_write_mm (board, C1_ALUTADR_OFFSET, 0x00);
  c1_write_mm (board, C1_BLUTADR_OFFSET, 0x00);
  c1_write_mm (board, C1_CLUTADR_OFFSET, 0x00);

  /* set start address of the matrix coefficients for Auto-INC */  
  c1_write_mm (board, C1_CSCADRLO_OFFSET, C1_MATRIX_LO);
  c1_write_mm (board, C1_CSCADRHI_OFFSET, C1_MATRIX_HI);   
  for (i=0; i<20; i++) {
    /* set matrix coefficients, cmd0 and cmd1 register to zero */
    c1_write_mm (board, C1_CSCDAT_OFFSET, 0x00);
  }
  
  if (c1_check_init (board) < 0) {
    icp.errno[board] = -EICP_MASK;
    return (ERROR);
  }

  return (OK);
}


/*
 * Miscellanous functions
 */

/* --- Output Lookup Table --- */

int 
c1_linearize_olut (u_char board) {

  int i, j, k;
  struct c1_olut_t *olut;

  if ((olut = vmalloc (sizeof (struct c1_olut_t))) == NULL) {
    icp.errno[board] = -EICP_POINTER;
    return (ERROR);
  }
 
  /* start to linearize first section of Output LUT A */
  k = 0;
  olut->olut_name = 'A';
  olut->olut_section = 0;
  /* generate linear Output LUT A */
  for (i=0; i<256; i++) {  
    for (j=0; j<128; j++) {
      olut->olut_data[k++] = i;
    }
    if ((i==63)||(i==127)||(i==191)||(i==255)) { 
      /* linearize this section of Output LUT A */   
      c1_set_olut (board, olut);   
      /* start to linearize next section of Output LUT A */
      olut->olut_section++; 
      k=0;
    }
  }

  /* generate linear output lookup table B */
  k = 0;
  for (i=0; i<8; i++) { 
    for (j=0; j<256; j++) {
      olut->olut_data[k++] = j;
    }
  }
  /* linearize output lookup table B */
  olut->olut_name = 'B';    
  olut->olut_section = 0;
  c1_set_olut (board, olut);

  /* start to linearize first section of Output LUT C */
  k = 0;
  olut->olut_name = 'C';   
  olut->olut_section = 0;
  /* generate linear Output LUT C */
  for (i=0; i<256; i++) {  
    for (j=0; j<128; j++) {
      olut->olut_data[k++] = i;  
    }
    if ((i==63)||(i==127)||(i==191)||(i==255)) { 
      /* linearize this section of Output LUT C */        
      c1_set_olut (board, olut);  
      /* start to linearize next section of Output LUT C */
      olut->olut_section++; 
      k=0;
    }
  }
  
  vfree (olut);

  return (OK);
}


/* --- Input Lookup Table --- */

int 
c1_linearize_ilut (u_char board) {

  int i;
  struct c1_ilut_t *ilut;

  if ((ilut = vmalloc (sizeof (struct c1_ilut_t))) == NULL) {
    icp.errno[board] = -EICP_POINTER;
    return (ERROR);
  }

  /* generate linear input lookup table */
  for (i=0; i<256; i++) ilut->ilut_data[i] = i;

  /* linearize input lookup table A */
  ilut->ilut_name = 'A';
  c1_set_ilut (board, ilut);
 
  /* linearize input lookup table B */  
  ilut->ilut_name = 'B';
  c1_set_ilut (board, ilut);

  /* linearize input lookup table C */
  ilut->ilut_name = 'C';
  c1_set_ilut (board, ilut);

  vfree (ilut);

  return (OK);
}


/* --- make register dump --- */

#ifdef DEBUG_ICPCI
void
c1_dump_regs (u_char board) {

  int count;
 
  for (count=0; count < C1_REG_SIZE; count++) {
    if (strcmp (c1_reg[count], "error   ") == 0) {
      icp_printf ("icpci[%d]: %#02x Reserved\n", board, count);
    } else {
      icp_printf ("icpci[%d]: %s %#04x\n", board, c1_reg[count],
		  c1_read_mm (board, count));
    }  
  }
}
#endif


/*
 * Functions used only in amclrReg.c
 */

/* convert a floating point value into a matrix coefficient */
/* used in : c1_csc_set (u_char, struct c1_csc_t *) */

u_short 
c1_float2matrix(float value) {

  short matrix;

  matrix = (short) ((value * 1024) / 4.000);
  if (matrix < -1024) matrix = -1024;
  if (matrix > 1023) matrix = 1023;
	 
  return (u_short) matrix & 0x7FF;
}

/* convert a matrix coefficient into a floating point value */
/* used in : c1_csc_get (u_char, struct c1_csc_t *) */

float 
c1_matrix2float (u_short matrix) { 

  short value;

  value = (short) matrix;
  if (value & 0x400) value = -((value ^ 0x7FF) + 1); 

  return ((float) value * (float) 4.000) / 1024;
}

/* check if all valid bits of the register are set to zero */  
/* used in : c1_init_regs (u_char) */

int 
c1_check_init (u_char board) {
  
  int i, error_count = 0;

  if ((0x3F & c1_read_mm (board, C1_CAMCON_OFFSET)) != 0) error_count++;
  if ((0xFF & c1_read_mm (board, C1_MODCON_OFFSET)) != 0) error_count++;
  if ((0xFD & c1_read_mm (board, C1_ADCCON_OFFSET)) != 0) error_count++;

  ICP_PRINTF("icpci[%d]: c1_ckeck_init: %d errors (CONTROL)\n", 
	     board, error_count);

  if ((0xFC & c1_read_mm (board, C1_RNREF_OFFSET)) != 0) error_count++;
  if ((0xFC & c1_read_mm (board, C1_RPREF_OFFSET)) != 0) error_count++;

  if ((0xFC & c1_read_mm (board, C1_GNREF_OFFSET)) != 0) error_count++;
  if ((0xFC & c1_read_mm (board, C1_GPREF_OFFSET)) != 0) error_count++;

  if ((0xFC & c1_read_mm (board, C1_BNREF_OFFSET)) != 0) error_count++;
  if ((0xFC & c1_read_mm (board, C1_BPREF_OFFSET)) != 0) error_count++;

  ICP_PRINTF("icpci[%d]: c1_check_init: %d errors (REF)\n",
	     board, error_count);

  if ((0xFF & c1_read_mm (board, C1_ALUTADR_OFFSET)) != 0) error_count++;
  if ((0x0F & c1_read_mm (board, C1_BLUTADR_OFFSET)) != 0) error_count++;
  if ((0xFF & c1_read_mm (board, C1_CLUTADR_OFFSET)) != 0) error_count++;

  ICP_PRINTF("icpci[%d]: c1_check_init: %d errors (LUTADR)\n",
	     board, error_count);

  /* set start address of the matrix coefficients for Auto-INC */
  c1_write_mm (board, C1_CSCADRLO_OFFSET, C1_MATRIX_LO);
  c1_write_mm (board, C1_CSCADRHI_OFFSET, C1_MATRIX_HI);   
  for (i=0; i<9; i++) {
    /* check matrix coefficient registers */
    if ((0xFF & c1_read_mm (board, C1_CSCDAT_OFFSET)) != 0) error_count++;
    if ((0x07 & c1_read_mm (board, C1_CSCDAT_OFFSET)) != 0) error_count++;
  }

  /* check CMD0 register */
  if ((0xFC & c1_read_mm (board, C1_CSCDAT_OFFSET)) != 0) error_count++;
  /* check CMD1 register */
  if ((0xE1 & c1_read_mm (board, C1_CSCDAT_OFFSET)) != 0) error_count++;

  ICP_PRINTF("icpci[%d]: c1_ckeck_init: %d errors (CSC)\n",
	     board, error_count);

  if (error_count != 0) return (ERROR);

  return (OK);
}
